sub mhr_scanner {
  #Team Cymru Malware Hash Registry (MHR) project

  use Digest::SHA;
  
  use Net::DNS;
  use IO::Select;

  my($start_mhr_time)=[gettimeofday];
  my($keep_scanning, $response, $virus_name, $stop_mhr_time, $mhr_time, $path);
  my($file,$result,$avHits,$lastSeen);
  
  &debug("mhr: starting scan of directory \"$ENV{'TMPDIR'}\"...");

  opendir(DIR, $ENV{'TMPDIR'});
  my @dirContents = readdir(DIR);
  closedir(DIR);

  my $sha = Digest::SHA->new("1");
  foreach $file (@dirContents) {
   if($file eq "." || $file eq ".." || $file =~ /^[0-9]+\.[0-9]+\-[0-9]+\.$hostname|^(orig\-|)$file_id|^textfile[0-9]+/){
     next;
   }
   $sha->addfile($file);
   my $digest = $sha->hexdigest;
   debug("mhr: digest of $file is $digest");  
   $result=&MHRviaDNS($digest);
   last if ($result);
  }

  if($result !~ /^([0-9]+):(.*)$/){
    &debug("mhr: $ENV{'TMPDIR'} is clean!\n");
  } else{
    $lastSeen=$1;$avHits=$2;
    &debug("mhr: $file scored $avHits");
    if ($avHits < $MHR_Trigger){
     &debug("mhr: $file scored too low - assume clean");
    }else{
     $quarantine_description="MHR-$avHits";
     ($quarantine_event=$quarantine_description)=~s/\s/_/g;
     &debug("There be a virus! ($quarantine_description)");
     $quarantine_event="MHR:".substr($quarantine_event,0,$QE_LEN);
     $description .= "\n---mhr results ---\n";
     $description .= "FILE/DIR INFECTED : $ENV{'TMPDIR'}\n";
     $description .= "VIRUS FOUND       : $quarantine_description\n";
    }
  }

  $stop_mhr_time=[gettimeofday];
  $mhr_time = tv_interval ($start_mhr_time, $stop_mhr_time);
  &debug("mhr: finished scan of dir \"$ENV{'TMPDIR'}\" in $mhr_time secs");
}

sub MHRviaDNS {
 my($hash)=@_;

 my $mhr_dns='malware.hash.cymru.com';

 my $res     = Net::DNS::Resolver->new(udp_timeout=> $MHRtimeout);
 my $bgsock  = $res->bgsend("${hash}.$mhr_dns","TXT");
 my $sel     = IO::Select->new($bgsock);

 my $dnsValue;

 my @ready = $sel->can_read($MHRtimeout);
 if (@ready) {
    foreach my $sock (@ready) {
      if ($sock == $bgsock) {
        my $packet = $res->bgread($bgsock);
        my $string=$packet->string;
        #this probably doesn't handle large DNS returns?
        if ($string=~/^${hash}.$mhr_dns.\s+[0-9]+\s+IN\s+TXT\s+\"([0-9]+)\s+([0-9]+)/mi){
             my $lastSeen=$1;
             my $avHits=$2;
	     return "$lastSeen:$avHits";
        }
        $bgsock = undef;
      }
      # Check for the other sockets.
      $sel->remove($sock);
      $sock = undef;
    }
  } else {
    &debug("MHRviaDNS: timed out after $MHRtimeout seconds");
  }
}
